# For this example, we're going to write a simple momentum script.  When the 
# stock goes up quickly, we're going to buy; when it goes down quickly, we're
# going to sell.  Hopefully we'll ride the waves.
import os
import tempfile
import time

import pandas as pd
import talib
from logbook import Logger

from catalyst import run_algorithm
from catalyst.api import symbol, record, order_target_percent, get_open_orders
from catalyst.exchange.stats_utils import extract_transactions
# We give a name to the algorithm which Catalyst will use to persist its state.
# In this example, Catalyst will create the `.catalyst/data/live_algos`
# directory. If we stop and start the algorithm, Catalyst will resume its
# state using the files included in the folder.
from catalyst.utils.paths import ensure_directory

NAMESPACE = 'mean_reversion_simple'
log = Logger(NAMESPACE)


# To run an algorithm in Catalyst, you need two functions: initialize and
# handle_data.

def initialize(context):
    # This initialize function sets any data or variables that you'll use in
    # your algorithm.  For instance, you'll want to define the trading pair (or
    # trading pairs) you want to backtest.  You'll also want to define any
    # parameters or values you're going to use.

    # In our example, we're looking at Ether in USD Tether.
    context.neo_eth = symbol('neo_eth')
    context.base_price = None
    context.current_day = None

    context.RSI_OVERSOLD = 50
    context.RSI_OVERBOUGHT = 80
    context.CANDLE_SIZE = '5T'

    context.start_time = time.time()


def handle_data(context, data):
    # This handle_data function is where the real work is done.  Our data is
    # minute-level tick data, and each minute is called a frame.  This function
    # runs on each frame of the data.

    # We flag the first period of each day.
    # Since cryptocurrencies trade 24/7 the `before_trading_starts` handle
    # would only execute once. This method works with minute and daily
    # frequencies.
    today = data.current_dt.floor('1D')
    if today != context.current_day:
        context.traded_today = False
        context.current_day = today

    # We're computing the volume-weighted-average-price of the security
    # defined above, in the context.neo_eth variable.  For this example, we're 
    # using three bars on the 15 min bars.

    # The frequency attribute determine the bar size. We use this convention
    # for the frequency alias:
    # http://pandas.pydata.org/pandas-docs/stable/timeseries.html#offset-aliases
    prices = data.history(
        context.neo_eth,
        fields='close',
        bar_count=50,
        frequency=context.CANDLE_SIZE
    )

    # Ta-lib calculates various technical indicator based on price and
    # volume arrays.

    # In this example, we are comp
    rsi = talib.RSI(prices.values, timeperiod=14)

    # We need a variable for the current price of the security to compare to
    # the average. Since we are requesting two fields, data.current()
    # returns a DataFrame with
    current = data.current(context.neo_eth, fields=['close', 'volume'])
    price = current['close']

    # If base_price is not set, we use the current value. This is the
    # price at the first bar which we reference to calculate price_change.
    if context.base_price is None:
        context.base_price = price

    price_change = (price - context.base_price) / context.base_price
    cash = context.portfolio.cash

    # Now that we've collected all current data for this frame, we use
    # the record() method to save it. This data will be available as
    # a parameter of the analyze() function for further analysis.
    record(
        price=price,
        volume=current['volume'],
        price_change=price_change,
        rsi=rsi[-1],
        cash=cash
    )

    # We are trying to avoid over-trading by limiting our trades to
    # one per day.
    if context.traded_today:
        return

    # Since we are using limit orders, some orders may not execute immediately
    # we wait until all orders are executed before considering more trades.
    orders = get_open_orders(context.neo_eth)
    if len(orders) > 0:
        return

    # Exit if we cannot trade
    if not data.can_trade(context.neo_eth):
        return

    # Another powerful built-in feature of the Catalyst backtester is the
    # portfolio object.  The portfolio object tracks your positions, cash,
    # cost basis of specific holdings, and more.  In this line, we calculate
    # how long or short our position is at this minute.   
    pos_amount = context.portfolio.positions[context.neo_eth].amount

    if rsi[-1] <= context.RSI_OVERSOLD and pos_amount == 0:
        log.info(
            '{}: buying - price: {}, rsi: {}'.format(
                data.current_dt, price, rsi[-1]
            )
        )
        # Set a style for limit orders,
        limit_price = price * 1.005
        order_target_percent(
            context.neo_eth, 1, limit_price=limit_price
        )
        context.traded_today = True

    elif rsi[-1] >= context.RSI_OVERBOUGHT and pos_amount > 0:
        log.info(
            '{}: selling - price: {}, rsi: {}'.format(
                data.current_dt, price, rsi[-1]
            )
        )
        limit_price = price * 0.995
        order_target_percent(
            context.neo_eth, 0, limit_price=limit_price
        )
        context.traded_today = True


def analyze(context=None, perf=None):
    end = time.time()
    log.info('elapsed time: {}'.format(end - context.start_time))

    import matplotlib.pyplot as plt
    # The base currency of the algo exchange
    base_currency = context.exchanges.values()[0].base_currency.upper()

    # Plot the portfolio value over time.
    ax1 = plt.subplot(611)
    perf.loc[:, 'portfolio_value'].plot(ax=ax1)
    ax1.set_ylabel('Portfolio Value ({})'.format(base_currency))

    # Plot the price increase or decrease over time.
    ax2 = plt.subplot(612, sharex=ax1)
    perf.loc[:, 'price'].plot(ax=ax2, label='Price')

    ax2.set_ylabel('{asset} ({base})'.format(
        asset=context.neo_eth.symbol, base=base_currency
    ))

    transaction_df = extract_transactions(perf)
    if not transaction_df.empty:
        buy_df = transaction_df[transaction_df['amount'] > 0]
        sell_df = transaction_df[transaction_df['amount'] < 0]
        ax2.scatter(
            buy_df.index.to_pydatetime(),
            perf.loc[buy_df.index.floor('1 min'), 'price'],
            marker='^',
            s=100,
            c='green',
            label=''
        )
        ax2.scatter(
            sell_df.index.to_pydatetime(),
            perf.loc[sell_df.index.floor('1 min'), 'price'],
            marker='v',
            s=100,
            c='red',
            label=''
        )

    ax4 = plt.subplot(613, sharex=ax1)
    perf.loc[:, 'cash'].plot(
        ax=ax4, label='Base Currency ({})'.format(base_currency)
    )
    ax4.set_ylabel('Cash ({})'.format(base_currency))

    perf['algorithm'] = perf.loc[:, 'algorithm_period_return']

    ax5 = plt.subplot(614, sharex=ax1)
    perf.loc[:, ['algorithm', 'price_change']].plot(ax=ax5)
    ax5.set_ylabel('Percent Change')

    ax6 = plt.subplot(615, sharex=ax1)
    perf.loc[:, 'rsi'].plot(ax=ax6, label='RSI')
    ax6.axhline(70, color='darkgoldenrod')
    ax6.axhline(30, color='darkgoldenrod')

    if not transaction_df.empty:
        ax6.scatter(
            buy_df.index.to_pydatetime(),
            perf.loc[buy_df.index.floor('1 min'), 'rsi'],
            marker='^',
            s=100,
            c='green',
            label=''
        )
        ax6.scatter(
            sell_df.index.to_pydatetime(),
            perf.loc[sell_df.index.floor('1 min'), 'rsi'],
            marker='v',
            s=100,
            c='red',
            label=''
        )
    plt.legend(loc=3)

    # Show the plot.
    plt.gcf().set_size_inches(18, 8)
    plt.show()
    pass


if __name__ == '__main__':
    # The execution mode: backtest or live
    MODE = 'backtest'

    if MODE == 'backtest':
        folder = os.path.join(
            tempfile.gettempdir(), 'catalyst', NAMESPACE
        )
        ensure_directory(folder)

        timestr = time.strftime('%Y%m%d-%H%M%S')
        out = os.path.join(folder, '{}.p'.format(timestr))
        # catalyst run -f catalyst/examples/mean_reversion_simple.py -x poloniex -s 2017-10-1 -e 2017-11-10 -c usdt -n mean-reversion --data-frequency minute --capital-base 10000
        run_algorithm(
            capital_base=10000,
            data_frequency='daily',
            initialize=initialize,
            handle_data=handle_data,
            analyze=analyze,
            exchange_name='bitfinex',
            algo_namespace=NAMESPACE,
            base_currency='usd',
            start=pd.to_datetime('2017-10-01', utc=True),
            end=pd.to_datetime('2017-11-10', utc=True),
            output=out
        )
        log.info('saved perf stats: {}'.format(out))

    elif MODE == 'live':
        run_algorithm(
            capital_base=0.5,
            initialize=initialize,
            handle_data=handle_data,
            analyze=analyze,
            exchange_name='bittrex',
            live=True,
            algo_namespace=NAMESPACE,
            base_currency='eth',
            live_graph=False
        )



import re
from collections import defaultdict
from datetime import timedelta

import pandas as pd
import ccxt
import time
from logbook import Logger

from catalyst.constants import LOG_LEVEL
from catalyst.exchange.exchange import Exchange
from catalyst.exchange.exchange_bundle import ExchangeBundle
from catalyst.exchange.exchange_errors import InvalidHistoryFrequencyError

log = Logger('Poloniex', level=LOG_LEVEL)


class CCXT(Exchange):
    def __init__(self, exchange_name, key, secret, base_currency,
                 portfolio=None):
        log.debug('available exchanges:\n{}'.format(ccxt.exchanges))
        self.api = ccxt.poloniex({
            'apiKey': key,
            'secret': secret,
        })
        markets = self.api.load_markets()
        log.debug('the markets:\n{}'.format(markets))

        self.name = exchange_name
        self.assets = {}
        self.load_assets()
        self.base_currency = base_currency
        self._portfolio = portfolio
        self.transactions = defaultdict(list)

        self.num_candles_limit = 2000
        self.max_requests_per_minute = 60
        self.request_cpt = dict()

        self.bundle = ExchangeBundle(self.name)

    def account(self):
        return None

    def time_skew(self):
        return None

    def get_symbol(self, asset):
        parts = asset.symbol.split('_')
        return '{}/{}'.format(parts[0].upper(), parts[1].upper())

    def get_timeframe(self, freq):
        freq_match = re.match(r'([0-9].*)?(m|M|d|D|h|H|T)', freq, re.M | re.I)
        if freq_match:
            candle_size = int(freq_match.group(1)) \
                if freq_match.group(1) else 1

            unit = freq_match.group(2)

        else:
            raise InvalidHistoryFrequencyError(frequency=freq)

        if unit.lower() == 'd':
            timeframe = '{}d'.format(candle_size)

        elif unit.lower() == 'm' or unit == 'T':
            timeframe = '{}m'.format(candle_size)

        elif unit.lower() == 'h' or unit == 'T':
            timeframe = '{}h'.format(candle_size)

        return timeframe

    def get_candles(self, freq, assets, bar_count=None, start_dt=None,
                    end_dt=None):
        symbols = self.get_symbols(assets)
        timeframe = self.get_timeframe(freq)
        delta = start_dt - pd.to_datetime('1970-1-1', utc=True)
        ms = int(delta.total_seconds()) * 1000

        ohlcvs = self.api.fetch_ohlcv(
            symbol=symbols[0],
            timeframe=timeframe,
            since=ms,
            limit=bar_count,
            params={}
        )

        candles = []
        for ohlcv in ohlcvs:
            candles.append(dict(
                last_traded=pd.to_datetime(ohlcv[0], unit='ms', utc=True),
                open=ohlcv[1],
                high=ohlcv[2],
                low=ohlcv[3],
                close=ohlcv[4],
                volume=ohlcv[5]
            ))
        return candles

    def get_balances(self):
        return None

    def create_order(self, asset, amount, is_buy, style):
        return None

    def get_open_orders(self, asset):
        return None

    def get_order(self, order_id):
        return None

    def cancel_order(self, order_param):
        return None

    def tickers(self, assets):
        return None

    def get_account(self):
        return None

    def get_orderbook(self, asset, order_type, limit):
        return None



from logbook import Logger

from catalyst.constants import LOG_LEVEL

log = Logger('AssetFinderExchange', level=LOG_LEVEL)


class AssetFinderExchange(object):
    def __init__(self):
        self._asset_cache = {}

    @property
    def sids(self):
        """
        This seems to be used to pre-fetch assets.
        I don't think that we need this for live-trading.
        Leaving the list empty.
        """
        return list()

    def retrieve_all(self, sids, default_none=False):
        """
        Retrieve all assets in `sids`.

        Parameters
        ----------
        sids : iterable of int
            Assets to retrieve.
        default_none : bool
            If True, return None for failed lookups.
            If False, raise `SidsNotFound`.

        Returns
        -------
        assets : list[Asset or None]
            A list of the same length as `sids` containing Assets (or Nones)
            corresponding to the requested sids.

        Raises
        ------
        SidsNotFound
            When a requested sid is not found and default_none=False.
        """
        # for sid in sids:
        #     if sid in self._asset_cache:
        #         log.debug('got asset from cache: {}'.format(sid))
        #     else:
        #         log.debug('fetching asset: {}'.format(sid))
        return list()

    def lookup_symbol(self, symbol, exchange, data_frequency=None,
                      as_of_date=None, fuzzy=False):
        """Lookup an asset by symbol.

        Parameters
        ----------
        symbol : str
            The ticker symbol to resolve.
        as_of_date : datetime or None
            Look up the last owner of this symbol as of this datetime.
            If ``as_of_date`` is None, then this can only resolve the equity
            if exactly one equity has ever owned the ticker.
        fuzzy : bool, optional
            Should fuzzy symbol matching be used? Fuzzy symbol matching
            attempts to resolve differences in representations for
            shareclasses. For example, some people may represent the ``A``
            shareclass of ``BRK`` as ``BRK.A``, where others could write
            ``BRK_A``.

        Returns
        -------
        equity : Asset
            The equity that held ``symbol`` on the given ``as_of_date``, or the
            only equity to hold ``symbol`` if ``as_of_date`` is None.

        Raises
        ------
        SymbolNotFound
            Raised when no equity has ever held the given symbol.
        MultipleSymbolsFound
            Raised when no ``as_of_date`` is given and more than one equity
            has held ``symbol``. This is also raised when ``fuzzy=True`` and
            there are multiple candidates for the given ``symbol`` on the
            ``as_of_date``.
        """
        log.debug('looking up symbol: {} {}'.format(symbol, exchange.name))

        if data_frequency is not None:
            key = ','.join([exchange.name, symbol, data_frequency])

        else:
            key = ','.join([exchange.name, symbol])

        if key in self._asset_cache:
            return self._asset_cache[key]
        else:
            asset = exchange.get_asset(symbol, data_frequency)
            self._asset_cache[key] = asset
            return asset



import json
import json
import time
from collections import defaultdict

import numpy as np
import pandas as pd
import pytz
from catalyst.assets._assets import TradingPair
from logbook import Logger
# import six
from six import iteritems

from catalyst.constants import LOG_LEVEL
# from websocket import create_connection
from catalyst.exchange.exchange import Exchange
from catalyst.exchange.exchange_bundle import ExchangeBundle
from catalyst.exchange.exchange_errors import (
    ExchangeRequestError,
    InvalidHistoryFrequencyError,
    InvalidOrderStyle, OrphanOrderReverseError)
from catalyst.exchange.exchange_execution import ExchangeLimitOrder, \
    ExchangeStopLimitOrder
from catalyst.exchange.exchange_utils import get_exchange_symbols_filename, \
    download_exchange_symbols, get_symbols_string
from catalyst.exchange.poloniex.poloniex_api import Poloniex_api
from catalyst.finance.order import Order, ORDER_STATUS
from catalyst.finance.transaction import Transaction
from catalyst.protocol import Account

log = Logger('Poloniex', level=LOG_LEVEL)


class Poloniex(Exchange):
    def __init__(self, key, secret, base_currency, portfolio=None):
        self.api = Poloniex_api(key=key, secret=secret)
        self.name = 'poloniex'

        self.assets = dict()
        self.load_assets()

        self.local_assets = dict()
        self.load_assets(is_local=True)

        self.base_currency = base_currency
        self._portfolio = portfolio
        self.minute_writer = None
        self.minute_reader = None
        self.transactions = defaultdict(list)

        self.num_candles_limit = 2000
        self.max_requests_per_minute = 60
        self.request_cpt = dict()

        self.bundle = ExchangeBundle(self.name)

    def sanitize_curency_symbol(self, exchange_symbol):
        """
        Helper method used to build the universal pair.
        Include any symbol mapping here if appropriate.

        :param exchange_symbol:
        :return universal_symbol:
        """
        return exchange_symbol.lower()

    def _create_order(self, order_status):
        """
        Create a Catalyst order object from the Exchange order dictionary
        :param order_status:
        :return: Order
        """
        # if order_status['is_cancelled']:
        #    status = ORDER_STATUS.CANCELLED
        # elif not order_status['is_live']:
        #    log.info('found executed order {}'.format(order_status))
        #    status = ORDER_STATUS.FILLED
        # else:
        status = ORDER_STATUS.OPEN

        amount = float(order_status['amount'])
        # filled = float(order_status['executed_amount'])
        filled = None

        if order_status['type'] == 'sell':
            amount = -amount
            # filled = -filled

        price = float(order_status['rate'])
        order_type = order_status['type']

        stop_price = None
        limit_price = None

        # TODO: is this comprehensive enough?
        # if order_type.endswith('limit'):
        #    limit_price = price
        # elif order_type.endswith('stop'):
        #    stop_price = price

        # executed_price = float(order_status['avg_execution_price'])
        executed_price = price

        # TODO: bitfinex does not specify comission. I could calculate it but not sure if it's worth it.
        commission = None

        # date = pd.Timestamp.utcfromtimestamp(float(order_status['timestamp']))
        # date = pytz.utc.localize(date)
        date = None

        order = Order(
            dt=date,
            asset=self.assets[order_status['symbol']],
            # No such field in Poloniex
            amount=amount,
            stop=stop_price,
            limit=limit_price,
            filled=filled,
            id=str(order_status['orderNumber']),
            commission=commission
        )
        order.status = status

        return order, executed_price

    def get_balances(self):
        balances = self.api.returnbalances()
        try:
            log.debug('retrieving wallets balances')
        except Exception as e:
            log.debug(e)
            raise ExchangeRequestError(error=e)

        if 'error' in balances:
            raise ExchangeRequestError(
                error='unable to fetch balance {}'.format(balances['error'])
            )

        std_balances = dict()
        for (key, value) in iteritems(balances):
            currency = key.lower()
            std_balances[currency] = float(value)

        return std_balances

    @property
    def account(self):
        account = Account()

        account.settled_cash = None
        account.accrued_interest = None
        account.buying_power = None
        account.equity_with_loan = None
        account.total_positions_value = None
        account.total_positions_exposure = None
        account.regt_equity = None
        account.regt_margin = None
        account.initial_margin_requirement = None
        account.maintenance_margin_requirement = None
        account.available_funds = None
        account.excess_liquidity = None
        account.cushion = None
        account.day_trades_remaining = None
        account.leverage = None
        account.net_leverage = None
        account.net_liquidation = None

        return account

    @property
    def time_skew(self):
        # TODO: research the time skew conditions
        return pd.Timedelta('0s')

    def get_account(self):
        # TODO: fetch account data and keep in cache
        return None

    def get_candles(self, freq, assets, bar_count=None,
                    start_dt=None, end_dt=None):
        """
        Retrieve OHLVC candles from Poloniex

        :param freq:
        :param assets:
        :param bar_count:
        :return:

        Available Frequencies
        ---------------------
        '5m', '15m', '30m', '2h', '4h', '1D'
        """

        if end_dt is None:
            end_dt = pd.Timestamp.utcnow()

        log.debug(
            'retrieving {bars} {freq} candles on {exchange} from '
            '{end_dt} for markets {symbols}, '.format(
                bars=bar_count,
                freq=freq,
                exchange=self.name,
                end_dt=end_dt,
                symbols=get_symbols_string(assets)
            )
        )

        if freq == '1T' and (bar_count == 1 or bar_count is None):
            # TODO: use the order book instead
            # We use the 5m to fetch the last bar
            frequency = 300
        elif freq == '5T':
            frequency = 300
        elif freq == '15T':
            frequency = 900
        elif freq == '30T':
            frequency = 1800
        elif freq == '120T':
            frequency = 7200
        elif freq == '240T':
            frequency = 14400
        elif freq == '1D':
            frequency = 86400
        else:
            # Poloniex does not offer 1m data candles
            # It is likely to error out there frequently
            raise InvalidHistoryFrequencyError(frequency=freq)

        # Making sure that assets are iterable
        asset_list = [assets] if isinstance(assets, TradingPair) else assets
        ohlc_map = dict()

        for asset in asset_list:
            delta = end_dt - pd.to_datetime('1970-1-1', utc=True)
            end = int(delta.total_seconds())

            if bar_count is None:
                start = end - 2 * frequency
            else:
                start = end - bar_count * frequency

            try:
                response = self.api.returnchartdata(
                    self.get_symbol(asset), frequency, start, end
                )
            except Exception as e:
                raise ExchangeRequestError(error=e)

            if 'error' in response:
                raise ExchangeRequestError(
                    error='Unable to retrieve candles: {}'.format(
                        response.content)
                )

            def ohlc_from_candle(candle):
                last_traded = pd.Timestamp.utcfromtimestamp(candle['date'])
                last_traded = last_traded.replace(tzinfo=pytz.UTC)

                ohlc = dict(
                    open=np.float64(candle['open']),
                    high=np.float64(candle['high']),
                    low=np.float64(candle['low']),
                    close=np.float64(candle['close']),
                    volume=np.float64(candle['volume']),
                    price=np.float64(candle['close']),
                    last_traded=last_traded
                )

                return ohlc

            if bar_count is None:
                ohlc_map[asset] = ohlc_from_candle(response[0])
            else:
                ohlc_bars = []
                for candle in response:
                    ohlc = ohlc_from_candle(candle)
                    ohlc_bars.append(ohlc)
                ohlc_map[asset] = ohlc_bars

        return ohlc_map[assets] \
            if isinstance(assets, TradingPair) else ohlc_map

    def create_order(self, asset, amount, is_buy, style):
        """
        Creating order on the exchange.

        :param asset:
        :param amount:
        :param is_buy:
        :param style:
        :return:
        """
        exchange_symbol = self.get_symbol(asset)

        if isinstance(style, ExchangeLimitOrder) or isinstance(style,
                                                               ExchangeStopLimitOrder):
            if isinstance(style, ExchangeStopLimitOrder):
                log.warn('{} will ignore the stop price'.format(self.name))

            price = style.get_limit_price(is_buy)

            try:
                if (is_buy):
                    response = self.api.buy(exchange_symbol, amount, price)
                else:
                    response = self.api.sell(exchange_symbol, -amount, price)
            except Exception as e:
                raise ExchangeRequestError(error=e)

            date = pd.Timestamp.utcnow()

            if ('orderNumber' in response):
                order_id = str(response['orderNumber'])
                order = Order(
                    dt=date,
                    asset=asset,
                    amount=amount,
                    stop=style.get_stop_price(is_buy),
                    limit=style.get_limit_price(is_buy),
                    id=order_id
                )
                return order
            else:
                log.warn(
                    '{} order failed: {}'.format('buy' if is_buy else 'sell',
                                                 response['error']))
                return None
        else:
            raise InvalidOrderStyle(exchange=self.name,
                                    style=style.__class__.__name__)

    def get_open_orders(self, asset='all'):
        """Retrieve all of the current open orders.

        Parameters
        ----------
        asset : Asset
            If passed and not 'all', return only the open orders for the given
            asset instead of all open orders.

        Returns
        -------
        open_orders : dict[list[Order]] or list[Order]
            If 'all' is passed this will return a dict mapping Assets
            to a list containing all the open orders for the asset.
            If an asset is passed then this will return a list of the open
            orders for this asset.
        """

        return self.portfolio.open_orders

        """
            TODO: Why going to the exchange if we already have this info locally?
                  And why creating all these Orders if we later discard them?
        """

        try:
            if (asset == 'all'):
                response = self.api.returnopenorders('all')
            else:
                response = self.api.returnopenorders(self.get_symbol(asset))
        except Exception as e:
            raise ExchangeRequestError(error=e)

        if 'error' in response:
            raise ExchangeRequestError(
                error='Unable to retrieve open orders: {}'.format(
                    order_statuses['message'])
            )

        print(self.portfolio.open_orders)

        # TODO: Need to handle openOrders for 'all'
        orders = list()
        for order_status in response:
            order, executed_price = self._create_order(
                order_status)  # will Throw error b/c Polo doesn't track order['symbol']
            if asset is None or asset == order.sid:
                orders.append(order)

        return orders

    def get_order(self, order_id):
        """Lookup an order based on the order id returned from one of the
        order functions.

        Parameters
        ----------
        order_id : str
            The unique identifier for the order.

        Returns
        -------
        order : Order
            The order object.
        """

        try:
            order = self._portfolio.open_orders[order_id]
        except Exception as e:
            raise OrphanOrderError(order_id=order_id, exchange=self.name)

        return order

        # TODO: Need to decide whether we fetch orders locally or from exchnage
        # The code below is ignored

        try:
            response = self.api.returnopenorders(self.get_symbol(order.sid))
        except Exception as e:
            raise ExchangeRequestError(error=e)

        for o in response:
            if (int(o['orderNumber']) == int(order_id)):
                return order

        return None

    def cancel_order(self, order_param):
        """Cancel an open order.

        Parameters
        ----------
        order_param : str or Order
            The order_id or order object to cancel.
        """

        if (isinstance(order_param, Order)):
            order = order_param
        else:
            order = self._portfolio.open_orders[order_param]

        try:
            response = self.api.cancelorder(order.id)
        except Exception as e:
            raise ExchangeRequestError(error=e)

        if 'error' in response:
            log.info(
                'Unable to cancel order {order_id} on exchange {exchange} {error}.'.format(
                    order_id=order.id,
                    exchange=self.name,
                    error=response['error']
                ))

            # raise OrderCancelError(
            #    order_id=order.id,
            #    exchange=self.name,
            #    error=response['error']
            # )

        self.portfolio.remove_order(order)

    def tickers(self, assets):
        """
        Fetch ticket data for assets
        https://docs.bitfinex.com/v2/reference#rest-public-tickers

        :param assets:
        :return:
        """
        symbols = self.get_symbols(assets)

        log.debug('fetching tickers {}'.format(symbols))

        try:
            response = self.api.returnticker()
        except Exception as e:
            raise ExchangeRequestError(error=e)

        if 'error' in response:
            raise ExchangeRequestError(
                error='Unable to retrieve tickers: {}'.format(
                    response['error'])
            )

        ticks = dict()

        for index, symbol in enumerate(symbols):
            ticks[assets[index]] = dict(
                timestamp=pd.Timestamp.utcnow(),
                bid=float(response[symbol]['highestBid']),
                ask=float(response[symbol]['lowestAsk']),
                last_price=float(response[symbol]['last']),
                low=float(response[symbol]['lowestAsk']),
                # TODO: Polo does not provide low
                high=float(response[symbol]['highestBid']),
                # TODO: Polo does not provide high
                volume=float(response[symbol]['baseVolume']),
            )

        log.debug('got tickers {}'.format(ticks))
        return ticks

    def generate_symbols_json(self, filename=None, source_dates=False):
        symbol_map = {}

        if not source_dates:
            fn, r = download_exchange_symbols(self.name)
            with open(fn) as data_file:
                cached_symbols = json.load(data_file)

        response = self.api.returnticker()

        for exchange_symbol in response:
            base, market = self.sanitize_curency_symbol(exchange_symbol).split(
                '_')
            symbol = '{market}_{base}'.format(market=market, base=base)

            if (source_dates):
                start_date = self.get_symbol_start_date(exchange_symbol)
            else:
                try:
                    start_date = cached_symbols[exchange_symbol]['start_date']
                except KeyError as e:
                    start_date = time.strftime('%Y-%m-%d')

            try:
                end_daily = cached_symbols[exchange_symbol]['end_daily']
            except KeyError as e:
                end_daily = 'N/A'

            try:
                end_minute = cached_symbols[exchange_symbol]['end_minute']
            except KeyError as e:
                end_minute = 'N/A'

            symbol_map[exchange_symbol] = dict(
                symbol=symbol,
                start_date=start_date,
                end_daily=end_daily,
                end_minute=end_minute,
            )

        if (filename is None):
            filename = get_exchange_symbols_filename(self.name)

        with open(filename, 'w') as f:
            json.dump(symbol_map, f, sort_keys=True, indent=2,
                      separators=(',', ':'))

    def get_symbol_start_date(self, symbol):
        try:
            r = self.api.returnchartdata(symbol, 86400, pd.to_datetime(
                '2010-1-1').value // 10 ** 9)
        except Exception as e:
            raise ExchangeRequestError(error=e)

        return time.strftime('%Y-%m-%d', time.gmtime(int(r[0]['date'])))

    def check_open_orders(self):
        """
        Need to override this function for Poloniex:

        Loop through the list of open orders in the Portfolio object.
        Check if any transactions have been executed:
            If so, create a transaction and apply to the Portfolio.
        Check if the order is still open:
            If not, remove it from open orders

        :return:
        transactions: Transaction[]
        """
        transactions = list()
        if self.portfolio.open_orders:
            for order_id in list(self.portfolio.open_orders):

                order = self._portfolio.open_orders[order_id]
                log.debug('found open order: {}'.format(order_id))

                try:
                    order_open = self.get_order(order_id)
                except Exception as e:
                    raise ExchangeRequestError(error=e)

                if (order_open):
                    delta = pd.Timestamp.utcnow() - order.dt
                    log.info(
                        'order {order_id} still open after {delta}'.format(
                            order_id=order_id,
                            delta=delta)
                    )

                try:
                    response = self.api.returnordertrades(order_id)
                except Exception as e:
                    raise ExchangeRequestError(error=e)

                if ('error' in response):
                    if (not order_open):
                        raise OrphanOrderReverseError(order_id=order_id,
                                                      exchange=self.name)
                else:
                    for tx in response:
                        """
                            We maintain a list of dictionaries of transactions that correspond to
                            partially filled orders, indexed by order_id. Every time we query 
                            executed transactions from the exchange, we check if we had that 
                            transaction for that order already. If not, we process it.

                            When an order if fully filled, we flush the dict of transactions  
                            associated with that order.
                        """
                        if (not filter(
                                lambda item: item['order_id'] == tx['tradeID'],
                                self.transactions[order_id])):
                            log.debug(
                                'Got new transaction for order {}: amount {}, price {}'.format(
                                    order_id, tx['amount'], tx['rate']))
                            tx['amount'] = float(tx['amount'])
                            if (tx['type'] == 'sell'):
                                tx['amount'] = -tx['amount']
                            transaction = Transaction(
                                asset=order.asset,
                                amount=tx['amount'],
                                dt=pd.to_datetime(tx['date'], utc=True),
                                price=float(tx['rate']),
                                order_id=tx['tradeID'],
                                # it's a misnomer, but keeping it for compatibility
                                commission=float(tx['fee'])
                            )
                            self.transactions[order_id].append(transaction)
                            self.portfolio.execute_transaction(transaction)
                            transactions.append(transaction)

                    if (not order_open):
                        """
                            Since transactions have been executed individually
                            the only thing left to do is remove them from list of open_orders
                        """
                        del self.portfolio.open_orders[order_id]
                        del self.transactions[order_id]

        return transactions

    def get_orderbook(self, asset, order_type='all'):
        exchange_symbol = asset.exchange_symbol
        data = self.api.returnOrderBook(market=exchange_symbol)

        result = dict()
        for order_type in data:
            # TODO: filter by type
            if order_type != 'asks' and order_type != 'bids':
                continue

            result[order_type] = []
            for entry in data[order_type]:
                if len(entry) == 2:
                    result[order_type].append(
                        dict(
                            rate=float(entry[0]),
                            quantity=float(entry[1])
                        )
                    )
        return result



import pandas as pd

from catalyst.api import symbol
from catalyst.utils.run_algo import run_algorithm


def initialize(context):
    context.asset = symbol('btc_usdt')


def handle_data(context, data):
    pass


def analyze(context=None, results=None):
    pass


start = pd.to_datetime('2017-11-1', utc=True)
end = pd.to_datetime('2017-11-3', utc=True)
results = run_algorithm(initialize=initialize,
                        handle_data=handle_data,
                        analyze=analyze,
                        start=start,
                        end=end,
                        exchange_name='poloniex',
                        capital_base=100000,
                        data_frequency='minute')



import numbers

import numpy as np
import pandas as pd


def trend_direction(series):
    if series[-1] is np.nan or series[-1] is np.nan:
        return None

    if series[-1] > series[-2]:
        return 'up'
    else:
        return 'down'


def crossover(source, target):
    """
    The `x`-series is defined as having crossed over `y`-series if the value
    of `x` is greater than the value of `y` and the value of `x` was less than
    the value of `y` on the bar immediately preceding the current bar.

    Parameters
    ----------
    source: Series
    target: Series

    Returns
    -------
    bool

    """
    if isinstance(target, numbers.Number):
        if source[-1] is np.nan or source[-2] is np.nan \
                or target is np.nan:
            return False

        if source[-1] >= target > source[-2]:
            return True
        else:
            return False

    else:
        if source[-1] is np.nan or source[-2] is np.nan \
                or target[-1] is np.nan or target[-2] is np.nan:
            return False

        if source[-1] > target[-1] and source[-2] < target[-2]:
            return True
        else:
            return False


def crossunder(source, target):
    """
    The `x`-series is defined as having crossed under `y`-series if the value
    of `x` is less than the value of `y` and the value of `x` was greater than
    the value of `y` on the bar immediately preceding the current bar.

    Parameters
    ----------
    source: Series
    target: Series

    Returns
    -------
    bool

    """
    if isinstance(target, numbers.Number):
        if source[-1] is np.nan or source[-2] is np.nan \
                or target is np.nan:
            return False

        if source[-1] < target <= source[-2]:
            return True
        else:
            return False
    else:
        if source[-1] is np.nan or source[-2] is np.nan \
                or target[-1] is np.nan or target[-2] is np.nan:
            return False

        if source[-1] < target[-1] and source[-2] >= target[-2]:
            return True
        else:
            return False


def vwap(df):
    """
    Volume-weighted average price (VWAP) is a ratio generally used by
    institutional investors and mutual funds to make buys and sells so as not
    to disturb the market prices with large orders. It is the average share
    price of a stock weighted against its trading volume within a particular
    time frame, generally one day.

    Read more: Volume Weighted Average Price - VWAP
    https://www.investopedia.com/terms/v/vwap.asp#ixzz4xt922daE

    Parameters
    ----------
    df: pd.DataFrame

    Returns
    -------

    """
    if 'close' not in df.columns or 'volume' not in df.columns:
        raise ValueError('price data must include `volume` and `close`')

    vol_sum = np.nansum(df['volume'].values)

    try:
        ret = np.nansum(df['close'].values * df['volume'].values) / vol_sum
    except ZeroDivisionError:
        ret = np.nan

    return ret


def get_pretty_stats(stats_df, recorded_cols=None, num_rows=10):
    """
    Format and print the last few rows of a statistics DataFrame.
    See the pyfolio project for the data structure.

    Parameters
    ----------
    stats_df: DataFrame
    num_rows: int

    Returns
    -------
    str

    """
    stats_df.set_index('period_close', drop=True, inplace=True)
    stats_df.dropna(axis=1, how='all', inplace=True)

    pd.set_option('display.expand_frame_repr', False)
    pd.set_option('precision', 3)
    pd.set_option('display.width', 1000)
    pd.set_option('display.max_colwidth', 1000)

    columns = ['starting_cash', 'ending_cash', 'portfolio_value',
               'pnl', 'long_exposure', 'short_exposure', 'orders',
               'transactions', 'positions']

    if recorded_cols is not None:
        for column in recorded_cols:
            columns.append(column)

    def format_positions(positions):
        parts = []
        for position in positions:
            msg = '{amount:.2f}{market} cost basis {cost_basis:.4f}{base}'.format(
                amount=position['amount'],
                market=position['sid'].market_currency,
                cost_basis=position['cost_basis'],
                base=position['sid'].base_currency
            )
            parts.append(msg)
        return ', '.join(parts)

    formatters = {
        'orders': lambda orders: len(orders),
        'transactions': lambda transactions: len(transactions),
        'returns': lambda returns: "{0:.4f}".format(returns),
        'positions': format_positions
    }

    return stats_df.tail(num_rows).to_string(
        columns=columns,
        formatters=formatters
    )


def df_to_string(df):
    """
    Create a formatted str representation of the DataFrame.

    Parameters
    ----------
    df: DataFrame

    Returns
    -------
    str

    """
    pd.set_option('display.expand_frame_repr', False)
    pd.set_option('precision', 8)
    pd.set_option('display.width', 1000)
    pd.set_option('display.max_colwidth', 1000)

    return df.to_string()


def extract_transactions(perf):
    """
    Compute indexes for buy and sell transactions

    Parameters
    ----------
    perf: DataFrame
        The algo performance DataFrame.

    Returns
    -------
    DataFrame
        A DataFrame of transactions.

    """
    trans_list = perf.transactions.values
    all_trans = [t for sublist in trans_list for t in sublist]
    all_trans.sort(key=lambda t: t['dt'])

    transactions = pd.DataFrame(all_trans)
    if not transactions.empty:
        transactions.set_index('dt', inplace=True, drop=True)
    return transactions



import pandas as pd
from logbook import Logger, DEBUG

from catalyst import run_algorithm
from catalyst.api import (schedule_function, order_target_percent, symbol,
                          date_rules, get_open_orders, cancel_order, record,
                          set_commission, set_slippage)

log = Logger('rodrigo_1', level=DEBUG)
"""
The initialize function sets any data or variables that 
you'll use in your algorithm. 
It's only called once at the beginning of your algorithm.
"""


def initialize(context):
    # Select asset of interest
    context.asset = symbol('BTC_USD')

    # set_commission(TradingPairFeeSchedule(maker_fee=0.5, taker_fee=0.5))
    # set_slippage(TradingPairFixedSlippage(spread=0.5))
    # Set up a rebalance method to run every day
    schedule_function(rebalance, date_rule=date_rules.every_day())


"""
Rebalance function scheduled to run once per day.
"""


def rebalance(context, data):
    # To make market decisions, we're calculating the token's
    # moving average for the last 5 days.

    # We get the price history for the last 5 days.
    price_history = data.history(context.asset, fields='price', bar_count=5,
                                 frequency='1d')

    # Then we take an average of those 5 days.
    average_price = price_history.mean()

    # We also get the coin's current price.
    price = data.current(context.asset, 'price')

    # Cancel any outstanding orders
    orders = get_open_orders(context.asset) or []
    for order in orders:
        cancel_order(order)

    # If our coin is currently listed on a major exchange
    if data.can_trade(context.asset):
        # If the current price is 1% above the 5-day average price,
        # we open a long position. If the current price is below the
        # average price, then we want to close our position to 0 shares.
        if price > (1.01 * average_price):
            # Place the buy order (positive means buy, negative means sell)
            order_target_percent(context.asset, .99)
            log.info("Buying %s" % (context.asset.symbol))
        elif price < average_price:
            # Sell all of our shares by setting the target position to zero
            order_target_percent(context.asset, 0)
            log.info("Selling %s" % (context.asset.symbol))

    # Use the record() method to track up to five custom signals.
    # Record Apple's current price and the average price over the last
    # five days.
    cash = context.portfolio.cash
    leverage = context.account.leverage

    record(price=price, average_price=average_price, cash=cash,
           leverage=leverage)


def analyze(context=None, results=None):
    import matplotlib.pyplot as plt

    # Plot the portfolio and asset data.
    ax1 = plt.subplot(511)
    results[['portfolio_value']].plot(ax=ax1)
    ax1.set_ylabel('Portfolio Value (USD)')

    ax2 = plt.subplot(512, sharex=ax1)
    ax2.set_ylabel('{asset} (USD)'.format(asset=context.asset))
    (results[[
        'price',
    ]]).plot(ax=ax2)

    trans = results.ix[[t != [] for t in results.transactions]]
    buys = trans.ix[
        [t[0]['amount'] > 0 for t in trans.transactions]
    ]
    sells = trans.ix[
        [t[0]['amount'] < 0 for t in trans.transactions]
    ]

    ax2.plot(
        buys.index,
        results.price[buys.index],
        '^',
        markersize=10,
        color='g',
    )
    ax2.plot(
        sells.index,
        results.price[sells.index],
        'v',
        markersize=10,
        color='r',
    )

    ax3 = plt.subplot(513, sharex=ax1)
    results[['leverage']].plot(ax=ax3)
    ax3.set_ylabel('Leverage ')

    ax4 = plt.subplot(514, sharex=ax1)
    results[['cash']].plot(ax=ax4)
    ax4.set_ylabel('Cash (USD)')

    results[[
        'algorithm',
        'benchmark',
    ]] = results[[
        'algorithm_period_return',
        'benchmark_period_return',
    ]]

    ax5 = plt.subplot(515, sharex=ax1)
    results[[
        'algorithm',
        'benchmark',
    ]].plot(ax=ax5)
    ax5.set_ylabel('Percent Change')

    plt.legend(loc=3)

    # Show the plot.
    plt.gcf().set_size_inches(18, 8)
    plt.show()


run_algorithm(
    capital_base=100000,
    start=pd.to_datetime('2017-1-1', utc=True),
    end=pd.to_datetime('2017-10-22', utc=True),
    data_frequency='minute',
    initialize=initialize,
    handle_data=None,
    analyze=analyze,
    exchange_name='bitfinex',
    algo_namespace='rodrigo_1',
    base_currency='usd'
)



#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import pickle
import signal
import sys
from collections import deque
from datetime import timedelta
from os import listdir
from os.path import isfile, join
from time import sleep

import logbook
import pandas as pd
from catalyst.assets._assets import TradingPair

import catalyst.protocol as zp
from catalyst.algorithm import TradingAlgorithm
from catalyst.constants import LOG_LEVEL
from catalyst.errors import OrderInBeforeTradingStart
from catalyst.exchange.exchange_blotter import ExchangeBlotter
from catalyst.exchange.exchange_errors import (
    ExchangeRequestError,
    ExchangePortfolioDataError,
    ExchangeTransactionError,
    OrphanOrderError)
from catalyst.exchange.exchange_execution import ExchangeStopLimitOrder, \
    ExchangeLimitOrder, ExchangeStopOrder
from catalyst.exchange.exchange_utils import save_algo_object, get_algo_object, \
    get_algo_folder, get_algo_df, \
    save_algo_df
from catalyst.exchange.live_graph_clock import LiveGraphClock
from catalyst.exchange.simple_clock import SimpleClock
from catalyst.exchange.stats_utils import get_pretty_stats
from catalyst.finance.execution import MarketOrder
from catalyst.finance.performance.period import calc_period_stats
from catalyst.gens.tradesimulation import AlgorithmSimulator
from catalyst.utils.api_support import (
    api_method,
    disallowed_in_before_trading_start)
from catalyst.utils.input_validation import error_keywords, ensure_upper_case, \
    expect_types
from catalyst.utils.math_utils import round_nearest
from catalyst.utils.preprocess import preprocess

log = logbook.Logger('exchange_algorithm', level=LOG_LEVEL)


class ExchangeAlgorithmExecutor(AlgorithmSimulator):
    def __init__(self, *args, **kwargs):
        super(self.__class__, self).__init__(*args, **kwargs)


class ExchangeTradingAlgorithmBase(TradingAlgorithm):
    def __init__(self, *args, **kwargs):
        self.exchanges = kwargs.pop('exchanges', None)

        super(ExchangeTradingAlgorithmBase, self).__init__(*args, **kwargs)

    def round_order(self, amount, asset):
        """
        We need fractions with cryptocurrencies

        :param amount:
        :return:
        """
        return round_nearest(amount, asset.min_trade_size)

    @api_method
    @preprocess(symbol_str=ensure_upper_case)
    def symbol(self, symbol_str, exchange_name=None):
        """Lookup an Equity by its ticker symbol.

        Parameters
        ----------
        symbol_str : str
            The ticker symbol for the equity to lookup.
        exchange_name: str
            The name of the exchange containing the symbol

        Returns
        -------
        equity : Equity
            The equity that held the ticker symbol on the current
            symbol lookup date.

        Raises
        ------
        SymbolNotFound
            Raised when the symbols was not held on the current lookup date.

        See Also
        --------
        :func:`catalyst.api.set_symbol_lookup_date`
        """
        # If the user has not set the symbol lookup date,
        # use the end_session as the date for sybmol->sid resolution.

        _lookup_date = self._symbol_lookup_date \
            if self._symbol_lookup_date is not None \
            else self.sim_params.end_session

        if exchange_name is None:
            exchange = list(self.exchanges.values())[0]
        else:
            exchange = self.exchanges[exchange_name]

        data_frequency = self.data_frequency \
            if self.sim_params.arena == 'backtest' else None
        return self.asset_finder.lookup_symbol(
            symbol=symbol_str,
            exchange=exchange,
            data_frequency=data_frequency,
            as_of_date=_lookup_date
        )

    def prepare_period_stats(self, start_dt, end_dt):
        """
        Creates a dictionary representing the state of the tracker.

        Parameters
        ----------
        start_dt: datetime
        end_dt: datetime

        Notes
        -----
        I rewrote this in an attempt to better control the stats.
        I don't want things to happen magically through complex logic
        pertaining to backtesting.

        """
        tracker = self.perf_tracker
        period = tracker.todays_performance

        pos_stats = period.position_tracker.stats()
        period_stats = calc_period_stats(pos_stats, period.ending_cash)

        stats = dict(
            period_start=tracker.period_start,
            period_end=tracker.period_end,
            capital_base=tracker.capital_base,
            progress=tracker.progress,
            ending_value=period.ending_value,
            ending_exposure=period.ending_exposure,
            capital_used=period.cash_flow,
            starting_value=period.starting_value,
            starting_exposure=period.starting_exposure,
            starting_cash=period.starting_cash,
            ending_cash=period.ending_cash,
            portfolio_value=period.ending_cash + period.ending_value,
            pnl=period.pnl,
            returns=period.returns,
            period_open=period.period_open,
            period_close=period.period_close,
            gross_leverage=period_stats.gross_leverage,
            net_leverage=period_stats.net_leverage,
            short_exposure=pos_stats.short_exposure,
            long_exposure=pos_stats.long_exposure,
            short_value=pos_stats.short_value,
            long_value=pos_stats.long_value,
            longs_count=pos_stats.longs_count,
            shorts_count=pos_stats.shorts_count,
        )

        # Merging cumulative risk
        stats.update(tracker.cumulative_risk_metrics.to_dict())

        # Merging latest recorded variables
        stats.update(self.recorded_vars)

        stats['positions'] = period.position_tracker.get_positions_list()

        # we want the key to be absent, not just empty
        # Only include transactions for given dt
        stats['transactions'] = []
        for date in period.processed_transactions:
            if start_dt <= date < end_dt:
                transactions = period.processed_transactions[date]
                for t in transactions:
                    stats['transactions'].append(t.to_dict())

        stats['orders'] = []
        for date in period.orders_by_modified:
            if start_dt <= date < end_dt:
                orders = period.orders_by_modified[date]
                for order in orders:
                    stats['orders'].append(orders[order].to_dict())

        return stats


class ExchangeTradingAlgorithmBacktest(ExchangeTradingAlgorithmBase):
    def __init__(self, *args, **kwargs):
        super(ExchangeTradingAlgorithmBacktest, self).__init__(*args, **kwargs)

        self.frame_stats = list()
        self.blotter = ExchangeBlotter(
            data_frequency=self.data_frequency,
            # Default to NeverCancel in catalyst
            cancel_policy=self.cancel_policy,
        )
        log.info('initialized trading algorithm in backtest mode')

    def _calculate_order(self, asset, amount,
                         limit_price=None, stop_price=None, style=None):
        # Raises a ZiplineError if invalid parameters are detected.
        self.validate_order_params(asset,
                                   amount,
                                   limit_price,
                                   stop_price,
                                   style)

        # Convert deprecated limit_price and stop_price parameters to use
        # ExecutionStyle objects.
        style = self.__convert_order_params_for_blotter(limit_price,
                                                        stop_price,
                                                        style)
        return amount, style

    @staticmethod
    def __convert_order_params_for_blotter(limit_price, stop_price, style):
        """
        Helper method for converting deprecated limit_price and stop_price
        arguments into ExecutionStyle instances.

        This function assumes that either style == None or (limit_price,
        stop_price) == (None, None).
        """
        if style:
            assert (limit_price, stop_price) == (None, None)
            return style
        if limit_price and stop_price:
            return ExchangeStopLimitOrder(limit_price, stop_price)
        if limit_price:
            return ExchangeLimitOrder(limit_price)
        if stop_price:
            return ExchangeStopOrder(stop_price)
        else:
            return MarketOrder()

    def is_last_frame_of_day(self, data):
        # TODO: adjust here to support more intervals
        next_frame_dt = data.current_dt + timedelta(minutes=1)
        if next_frame_dt.date() > data.current_dt.date():
            return True
        else:
            return False

    def handle_data(self, data):
        super(ExchangeTradingAlgorithmBacktest, self).handle_data(data)

        if self.data_frequency == 'minute':
            frame_stats = self.prepare_period_stats(
                data.current_dt, data.current_dt + timedelta(minutes=1)
            )
            self.frame_stats.append(frame_stats)

    def _create_stats_df(self):
        stats = pd.DataFrame(self.frame_stats)
        stats.set_index('period_close', inplace=True, drop=False)
        return stats

    def analyze(self, perf):
        stats = self._create_stats_df() if self.data_frequency == 'minute' \
            else perf
        super(ExchangeTradingAlgorithmBacktest, self).analyze(stats)

    def run(self, data=None, overwrite_sim_params=True):
        perf = super(ExchangeTradingAlgorithmBacktest, self).run(
            data, overwrite_sim_params
        )
        # Rebuilding the stats to support minute data
        stats = self._create_stats_df() if self.data_frequency == 'minute' \
            else perf
        return stats


class ExchangeTradingAlgorithmLive(ExchangeTradingAlgorithmBase):
    def __init__(self, *args, **kwargs):
        self.algo_namespace = kwargs.pop('algo_namespace', None)
        self.live_graph = kwargs.pop('live_graph', None)

        self._clock = None
        self.frame_stats = deque(maxlen=60)

        self.pnl_stats = get_algo_df(self.algo_namespace, 'pnl_stats')

        self.custom_signals_stats = \
            get_algo_df(self.algo_namespace, 'custom_signals_stats')

        self.exposure_stats = \
            get_algo_df(self.algo_namespace, 'exposure_stats')

        self.is_running = True

        self.retry_check_open_orders = 5
        self.retry_synchronize_portfolio = 5
        self.retry_get_open_orders = 5
        self.retry_order = 2
        self.retry_delay = 5

        self.stats_minutes = 5

        super(ExchangeTradingAlgorithmLive, self).__init__(*args, **kwargs)

        signal.signal(signal.SIGINT, self.signal_handler)

        log.info('initialized trading algorithm in live mode')

    def signal_handler(self, signal, frame):
        """
        Handles the keyboard interruption signal.

        Parameters
        ----------
        signal
        frame

        Returns
        -------

        """
        self.is_running = False

        if self._analyze is None:
            log.info('Interruption signal detected {}, exiting the '
                     'algorithm'.format(signal))

        else:
            log.info('Interruption signal detected {}, calling `analyze()` '
                     'before exiting the algorithm'.format(signal))

            algo_folder = get_algo_folder(self.algo_namespace)
            folder = join(algo_folder, 'daily_perf')
            files = [f for f in listdir(folder) if isfile(join(folder, f))]

            daily_perf_list = []
            for item in files:
                filename = join(folder, item)
                with open(filename, 'rb') as handle:
                    daily_perf_list.append(pickle.load(handle))

            stats = pd.DataFrame(daily_perf_list)

            self.analyze(stats)

        sys.exit(0)

    @property
    def clock(self):
        if self._clock is None:
            return self._create_clock()
        else:
            return self._clock

    def _create_clock(self):

        # The calendar's execution times are the minutes over which we actually
        # want to run the clock. Typically the execution times simply adhere to
        # the market open and close times. In the case of the futures calendar,
        # for example, we only want to simulate over a subset of the full 24
        # hour calendar, so the execution times dictate a market open time of
        # 6:31am US/Eastern and a close of 5:00pm US/Eastern.

        # In our case, we are trading around the clock, so the market close
        # corresponds to the last minute of the day.

        # This method is taken from TradingAlgorithm.
        # The clock has been replaced to use RealtimeClock
        # TODO: should we apply a time skew? not sure to understand the utility.

        log.debug('creating clock')
        if self.live_graph:
            self._clock = LiveGraphClock(
                self.sim_params.sessions,
                context=self
            )
        else:
            self._clock = SimpleClock(
                self.sim_params.sessions,
            )

        return self._clock

    def _create_generator(self, sim_params):
        if self.perf_tracker is None:
            self.perf_tracker = get_algo_object(
                algo_name=self.algo_namespace,
                key='perf_tracker'
            )

        # Call the simulation trading algorithm for side-effects:
        # it creates the perf tracker
        TradingAlgorithm._create_generator(self, sim_params)
        self.trading_client = ExchangeAlgorithmExecutor(
            self,
            sim_params,
            self.data_portal,
            self.clock,
            self._create_benchmark_source(),
            self.restrictions,
            universe_func=self._calculate_universe
        )

        return self.trading_client.transform()

    def updated_portfolio(self):
        """
        We skip the entire performance tracker business and update the
        portfolio directly.

        Returns
        -------
        ExchangePortfolio

        """
        # TODO: build cumulative portfolio
        return self.perf_tracker.get_portfolio(False)

    def updated_account(self):
        return self.perf_tracker.get_account(False)

    def _synchronize_portfolio(self, attempt_index=0):
        try:
            for exchange_name in self.exchanges:
                exchange = self.exchanges[exchange_name]

                exchange.synchronize_portfolio()

                # Applying the updated last_sales_price to the positions
                # in the performance tracker. This seems a bit redundant
                # but it will make sense when we have multiple exchange portfolios
                # feeding into the same performance tracker.
                tracker = self.perf_tracker.todays_performance.position_tracker
                for asset in exchange.portfolio.positions:
                    position = exchange.portfolio.positions[asset]
                    tracker.update_position(
                        asset=asset,
                        last_sale_date=position.last_sale_date,
                        last_sale_price=position.last_sale_price
                    )
        except ExchangeRequestError as e:
            log.warn(
                'update portfolio attempt {}: {}'.format(attempt_index, e)
            )
            if attempt_index < self.retry_synchronize_portfolio:
                sleep(self.retry_delay)
                self._synchronize_portfolio(attempt_index + 1)
            else:
                raise ExchangePortfolioDataError(
                    data_type='update-portfolio',
                    attempts=attempt_index,
                    error=e
                )

    def _check_open_orders(self, attempt_index=0):
        try:
            orders = list()
            for exchange_name in self.exchanges:
                exchange = self.exchanges[exchange_name]
                exchange_orders = exchange.check_open_orders()

                orders += exchange_orders

            return orders
        except ExchangeRequestError as e:
            log.warn(
                'check open orders attempt {}: {}'.format(attempt_index, e)
            )
            if attempt_index < self.retry_check_open_orders:
                sleep(self.retry_delay)
                return self._check_open_orders(attempt_index + 1)
            else:
                raise ExchangePortfolioDataError(
                    data_type='order-status',
                    attempts=attempt_index,
                    error=e
                )

    def add_pnl_stats(self, period_stats):
        """
        Save p&l stats.

        Parameters
        ----------
        period_stats

        Returns
        -------

        """
        starting = period_stats['starting_cash']
        current = period_stats['portfolio_value']
        appreciation = (current / starting) - 1
        perc = (appreciation * 100) if current != 0 else 0

        log.debug('adding pnl stats: {:6f}%'.format(perc))

        df = pd.DataFrame(
            data=[dict(performance=perc)],
            index=[period_stats['period_close']]
        )
        self.pnl_stats = pd.concat([self.pnl_stats, df])

        save_algo_df(self.algo_namespace, 'pnl_stats', self.pnl_stats)

    def add_custom_signals_stats(self, period_stats):
        """
        Save custom signals stats.

        Parameters
        ----------
        period_stats

        Returns
        -------

        """
        log.debug('adding custom signals stats: {}'.format(self.recorded_vars))
        df = pd.DataFrame(
            data=[self.recorded_vars],
            index=[period_stats['period_close']],
        )
        self.custom_signals_stats = pd.concat([self.custom_signals_stats, df])

        save_algo_df(self.algo_namespace, 'custom_signals_stats',
                     self.custom_signals_stats)

    def add_exposure_stats(self, period_stats):
        """
        Save exposure stats.

        Parameters
        ----------
        period_stats

        Returns
        -------

        """
        data = dict(
            long_exposure=period_stats['long_exposure'],
            base_currency=period_stats['ending_cash']
        )
        log.debug('adding exposure stats: {}'.format(data))

        df = pd.DataFrame(
            data=[data],
            index=[period_stats['period_close']],
        )
        self.exposure_stats = pd.concat([self.exposure_stats, df])

        save_algo_df(
            self.algo_namespace, 'exposure_stats', self.exposure_stats
        )

    def handle_data(self, data):
        """
        Wrapper around the handle_data method of each algo.

        Parameters
        ----------
        data

        """
        if not self.is_running:
            return

        self._synchronize_portfolio()

        transactions = self._check_open_orders()
        if len(transactions) > 0:
            for transaction in transactions:
                self.perf_tracker.process_transaction(transaction)

            self.perf_tracker.update_performance()

        if self._handle_data:
            self._handle_data(self, data)

        # Unlike trading controls which remain constant unless placing an
        # order, account controls can change each bar. Thus, must check
        # every bar no matter if the algorithm places an order or not.
        self.validate_account_controls()

        try:
            # Since the clock runs 24/7, I trying to disable the daily
            # Performance tracker and keep only minute and cumulative
            self.perf_tracker.update_performance()

            frame_stats = self.prepare_period_stats(
                data.current_dt, data.current_dt + timedelta(minutes=1))

            # Saving the last hour in memory
            self.frame_stats.append(frame_stats)

            self.add_pnl_stats(frame_stats)
            if self.recorded_vars:
                self.add_custom_signals_stats(frame_stats)
                recorded_cols = list(self.recorded_vars.keys())
            else:
                recorded_cols = None

            self.add_exposure_stats(frame_stats)

            print_df = pd.DataFrame(list(self.frame_stats))
            log.info(
                'statistics for the last {stats_minutes} minutes:\n{stats}'.format(
                    stats_minutes=self.stats_minutes,
                    stats=get_pretty_stats(
                        stats_df=print_df,
                        recorded_cols=recorded_cols,
                        num_rows=self.stats_minutes
                    )
                ))

            today = pd.to_datetime('today', utc=True)
            daily_stats = self.prepare_period_stats(
                start_dt=today,
                end_dt=pd.Timestamp.utcnow()
            )
            save_algo_object(
                algo_name=self.algo_namespace,
                key=today.strftime('%Y-%m-%d'),
                obj=daily_stats,
                rel_path='daily_perf'
            )

        except Exception as e:
            log.warn('unable to calculate performance: {}'.format(e))

        # TODO: pickle does not seem to work in python 3
        try:
            save_algo_object(
                algo_name=self.algo_namespace,
                key='perf_tracker',
                obj=self.perf_tracker
            )
        except Exception as e:
            log.warn('unable to save minute perfs to disk: {}'.format(e))

        try:
            for exchange_name in self.exchanges:
                exchange = self.exchanges[exchange_name]
                save_algo_object(
                    algo_name=self.algo_namespace,
                    key='portfolio_{}'.format(exchange_name),
                    obj=exchange.portfolio
                )
        except Exception as e:
            log.warn('unable to save portfolio to disk: {}'.format(e))

    def _order(self,
               asset,
               amount,
               limit_price=None,
               stop_price=None,
               style=None,
               attempt_index=0):
        try:
            exchange = self.exchanges[asset.exchange]
            return exchange.order(asset, amount, limit_price,
                                  stop_price,
                                  style)
        except ExchangeRequestError as e:
            log.warn(
                'order attempt {}: {}'.format(attempt_index, e)
            )
            if attempt_index < self.retry_order:
                sleep(self.retry_delay)
                return self._order(
                    asset, amount, limit_price, stop_price, style,
                    attempt_index + 1)
            else:
                raise ExchangeTransactionError(
                    transaction_type='order',
                    attempts=attempt_index,
                    error=e
                )

    @api_method
    @disallowed_in_before_trading_start(OrderInBeforeTradingStart())
    @expect_types(asset=TradingPair)
    def order(self,
              asset,
              amount,
              limit_price=None,
              stop_price=None,
              style=None):
        """
        We use the exchange specific portfolio to place orders.
        The cumulative portfolio does not contain open orders but exchange
        portfolios do.

        Parameters
        ----------
        asset: TradingPair
        amount: float
        limit_price: float
        stop_price: float
        style: Style
        order: Order
            The catalyst order object or None
        """
        amount, style = self._calculate_order(asset, amount,
                                              limit_price, stop_price,
                                              style)

        order_id = self._order(asset, amount, limit_price, stop_price, style)

        exchange = self.exchanges[asset.exchange]
        exchange_portfolio = exchange.portfolio
        if order_id is not None:

            if order_id in exchange_portfolio.open_orders:
                order = exchange_portfolio.open_orders[order_id]
                self.perf_tracker.process_order(order)
                return order

            else:
                raise OrphanOrderError(
                    order_id=order_id,
                    exchange=exchange.name
                )
        else:
            log.warn('unable to order {} {} on exchange {}'.format(
                amount, asset.symbol, asset.exchange))
            return None

    @api_method
    def batch_market_order(self, share_counts):
        raise NotImplementedError()

    def _get_open_orders(self, asset=None, attempt_index=0):
        try:
            if asset:
                exchange = self.exchanges[asset.exchange]
                return exchange.get_open_orders(asset)

            else:
                open_orders = []
                for exchange_name in self.exchanges:
                    exchange = self.exchanges[exchange_name]
                    exchange_orders = exchange.get_open_orders()
                    open_orders.append(exchange_orders)

                return open_orders
        except ExchangeRequestError as e:
            log.warn(
                'open orders attempt {}: {}'.format(attempt_index, e)
            )
            if attempt_index < self.retry_get_open_orders:
                sleep(self.retry_delay)
                return self._get_open_orders(asset, attempt_index + 1)
            else:
                raise ExchangePortfolioDataError(
                    data_type='open-orders',
                    attempts=attempt_index,
                    error=e
                )

    @error_keywords(sid='Keyword argument `sid` is no longer supported for '
                        'get_open_orders. Use `asset` instead.')
    @api_method
    def get_open_orders(self, asset=None):
        """Retrieve all of the current open orders.

        Parameters
        ----------
        asset : Asset
            If passed and not None, return only the open orders for the given
            asset instead of all open orders.

        Returns
        -------
        open_orders : dict[list[Order]] or list[Order]
            If no asset is passed this will return a dict mapping Assets
            to a list containing all the open orders for the asset.
            If an asset is passed then this will return a list of the open
            orders for this asset.
        """
        return self._get_open_orders(asset)

    @api_method
    def get_order(self, order_id, exchange_name):
        """Lookup an order based on the order id returned from one of the
        order functions.

        Parameters
        ----------
        order_id : str
            The unique identifier for the order.

        Returns
        -------
        order : Order
            The order object.
        execution_price: float
            The execution price per share of the order
        """
        exchange = self.exchanges[exchange_name]
        return exchange.get_order(order_id)

    @api_method
    def cancel_order(self, order_param, exchange_name):
        """Cancel an open order.

        Parameters
        ----------
        order_param : str or Order
            The order_id or order object to cancel.
        """
        exchange = self.exchanges[exchange_name]

        order_id = order_param
        if isinstance(order_param, zp.Order):
            order_id = order_param.id

        exchange.cancel_order(order_id)



import os
import tempfile

import pandas as pd
import six
from catalyst.assets._assets import TradingPair, get_calendar
from logbook import Logger
from pandas.util.testing import assert_frame_equal

from catalyst.constants import LOG_LEVEL
from catalyst.exchange.asset_finder_exchange import AssetFinderExchange
from catalyst.exchange.exchange_data_portal import DataPortalExchangeBacktest
from catalyst.exchange.factory import get_exchanges
from catalyst.utils.paths import ensure_directory

log = Logger('Validator', level=LOG_LEVEL)


def output_df(df, assets, name=None):
    """
    Outputs a price DataFrame to a temp folder.

    Parameters
    ----------
    df: pd.DataFrame
    assets
    name

    Returns
    -------

    """
    if isinstance(assets, TradingPair):
        exchange_folder = assets.exchange
        asset_folder = assets.symbol
    else:
        exchange_folder = ','.join([asset.exchange for asset in assets])
        asset_folder = ','.join([asset.symbol for asset in assets])

    folder = os.path.join(
        tempfile.gettempdir(), 'catalyst', exchange_folder, asset_folder
    )
    ensure_directory(folder)

    if name is None:
        name = 'output'

    path = os.path.join(folder, '{}.csv'.format(name))
    df.to_csv(path)

    return path


class Validator(object):
    def __init__(self, data_portal):
        self.data_portal = data_portal

    def compare_bundle_with_exchange(self, exchange, assets, end_dt, bar_count,
                                     sample_minutes):
        """
        Creates DataFrames from the bundle and exchange for the specified
        data set.

        Parameters
        ----------
        exchange: Exchange
        assets
        end_dt
        bar_count
        sample_minutes

        Returns
        -------

        """
        freq = '{}T'.format(sample_minutes)

        log.info('creating data sample from bundle')
        df1 = self.data_portal.get_history_window(
            assets=assets,
            end_dt=end_dt,
            bar_count=bar_count,
            frequency=freq,
            field='close',
            data_frequency='minute'
        )
        path = output_df(df1, assets, '{}_resampled'.format(freq))
        log.info('saved resampled bundle candles: {}\n{}'.format(
            path, df1.tail(10))
        )

        log.info('creating data sample from exchange api')
        candles = exchange.get_candles(
            end_dt=end_dt,
            freq='{}T'.format(sample_minutes),
            assets=assets,
            bar_count=bar_count
        )

        series = dict()
        for asset in assets:
            series[asset] = pd.Series(
                data=[candle['close'] for candle in candles[asset]],
                index=[candle['last_traded'] for candle in candles[asset]]
            )

        df2 = pd.DataFrame(series)
        path = output_df(df2, assets, '{}_api'.format(freq))
        log.info('saved exchange api candles: {}\n{}'.format(
            path, df2.tail(10))
        )

        try:
            assert_frame_equal(df1, df2)
            return True
        except:
            log.warn('differences found in dataframes')
            return False


if __name__ == '__main__':
    exchanges = get_exchanges(['poloniex'])
    exchange = six.next(six.itervalues(exchanges))
    assets = exchange.get_assets(symbols=['eth_btc'])

    open_calendar = get_calendar('OPEN')
    asset_finder = AssetFinderExchange()
    data_portal = DataPortalExchangeBacktest(
        exchanges=exchanges,
        asset_finder=asset_finder,
        trading_calendar=open_calendar,
        first_trading_day=None  # will set dynamically based on assets
    )
    validator = Validator(data_portal=data_portal)

    validator.compare_bundle_with_exchange(
        exchange=exchange,
        assets=assets,
        end_dt=pd.to_datetime('2017-11-10 1:00', utc=True),
        bar_count=200,
        sample_minutes=30
    )



import talib
import pandas as pd
from logbook import Logger

from catalyst.api import (
    order,
    order_target_percent,
    symbol,
    record,
    get_open_orders,
)
from catalyst.utils.run_algo import run_algorithm

algo_namespace = 'korean_rotation'
log = Logger(algo_namespace)


def initialize(context):
    log.info('initializing algo')

    context.etc_usd = symbol('etc_usd')
    context.ltc_usd = symbol('ltc_usd')
    context.xmr_usd = symbol('xmr_usd')
    context.xrp_usd = symbol('xrp_usd')
    context.eth_usd = symbol('eth_usd')
    context.bch_usd = symbol('bch_usd')
    context.dsh_usd = symbol('dsh_usd')

    context.last_rebalance = None
    context.ALLOWANCE_USD = 2000

    context.RSI_BUY_MAX = 40
    context.RSI_SELL_MIN = 70

    context.errors = []
    pass


def _handle_data(context, data):
    this_hour = pd.Timestamp.utcnow().floor('1h')
    if context.last_rebalance is None or this_hour > context.last_rebalance:
        log.debug('rebalancing portfolio')
    else:
        return

    context.last_rebalance = pd.Timestamp.utcnow()
    assets = [
        context.etc_usd,
        context.ltc_usd,
        context.xmr_usd,
        context.xrp_usd,
        context.eth_usd,
        context.bch_usd,
        context.dsh_usd
    ]
    asset_prices = data.history(
        assets,
        fields='price',
        bar_count=60,
        frequency='1h'
    )
    for asset in assets:
        prices = asset_prices[asset]
        rsi = talib.RSI(prices.values, timeperiod=14)[-1]
        macd, signal, hist = talib.MACD(
            prices.values, fastperiod=12, slowperiod=26, signalperiod=9)
        macd_val = macd[-1] - signal[-1]

        log.info('{} rsi: {} macd: {}'.format(asset.symbol, rsi, macd_val))


def handle_data(context, data):
    log.info('handling bar {}'.format(data.current_dt))
    # try:
    _handle_data(context, data)
    # except Exception as e:
    #     log.warn('aborting the bar on error {}'.format(e))
    #     context.errors.append(e)

    log.info('completed bar {}, total execution errors {}'.format(
        data.current_dt,
        len(context.errors)
    ))

    if len(context.errors) > 0:
        log.info('the errors:\n{}'.format(context.errors))


def analyze(context, stats):
    log.info('the full stats:\n{}'.format(stats.head()))
    pass


run_algorithm(
    initialize=initialize,
    handle_data=handle_data,
    analyze=analyze,
    exchange_name='bitfinex',
    live=True,
    algo_namespace=algo_namespace,
    base_currency='usd'
)



import pandas as pd
from catalyst.gens.sim_engine import (
    BAR,
    SESSION_START
)
from logbook import Logger

from catalyst.constants import LOG_LEVEL
from catalyst.exchange.exchange_errors import \
    MismatchingBaseCurrenciesExchanges

log = Logger('LiveGraphClock', level=LOG_LEVEL)


class LiveGraphClock(object):
    """Realtime clock for live trading.

    This class is a drop-in replacement for
    :class:`zipline.gens.sim_engine.MinuteSimulationClock`.

    This mixes the clock with a live graph.

    Notes
    -----
    This seemingly awkward approach allows us to run the program using a single
    thread. This is important because Matplotlib does not play nice with
    multi-threaded environments. Zipline probably does not either.


    Matplotlib has a pause() method which is a wrapper around time.sleep()
    used in the SimpleClock. The key difference is that users
    can still interact with the chart during the pause cycles. This is
    what enables us to keep a single thread. This is also why we are not using
    the 'animate' callback of Matplotlib. We need to direct access to the
    __iter__ method in order to yield events to Zipline.

    The :param:`time_skew` parameter represents the time difference between
    the exchange and the live trading machine's clock. It's not used currently.
    """

    def __init__(self, sessions, context, time_skew=pd.Timedelta('0s')):

        global mdates, plt  # TODO: Could be cleaner
        import matplotlib.dates as mdates
        from matplotlib import pyplot as plt
        from matplotlib import style

        self.sessions = sessions
        self.time_skew = time_skew
        self._last_emit = None
        self._before_trading_start_bar_yielded = True
        self.context = context
        self.fmt = mdates.DateFormatter('%Y-%m-%d %H:%M')

        style.use('dark_background')

        fig = plt.figure()
        fig.canvas.set_window_title('Enigma Catalyst: {}'.format(
            self.context.algo_namespace))

        self.ax_pnl = fig.add_subplot(311)

        self.ax_custom_signals = fig.add_subplot(312, sharex=self.ax_pnl)

        self.ax_exposure = fig.add_subplot(313, sharex=self.ax_pnl)

        if len(context.minute_stats) > 0:
            self.draw_pnl()
            self.draw_custom_signals()
            self.draw_exposure()

        # rotates and right aligns the x labels, and moves the bottom of the
        # axes up to make room for them
        fig.autofmt_xdate()
        fig.subplots_adjust(hspace=0.5)

        plt.tight_layout()
        plt.ion()
        plt.show()

    def format_ax(self, ax):
        """
        Trying to assign reasonable parameters to the time axis.

        Parameters
        ----------
        ax:

        """
        # TODO: room for improvement
        ax.xaxis.set_major_locator(mdates.DayLocator(interval=1))
        ax.xaxis.set_major_formatter(self.fmt)

        locator = mdates.HourLocator(interval=4)
        locator.MAXTICKS = 5000
        ax.xaxis.set_minor_locator(locator)

        datemin = pd.Timestamp.utcnow()
        ax.set_xlim(datemin)

        ax.grid(True)

    def set_legend(self, ax):
        """
        Set legend on the chart.

        Parameters
        ----------
        ax

        """
        ax.legend(loc='upper left', ncol=1, fontsize=10, numpoints=1)

    def draw_pnl(self):
        """
        Draw p&l line on the chart.

        """
        ax = self.ax_pnl
        df = self.context.pnl_stats

        ax.clear()
        ax.set_title('Performance')
        ax.plot(df.index, df['performance'], '-',
                color='green',
                linewidth=1.0,
                label='Performance'
                )

        def perc(val):
            return '{:2f}'.format(val)

        ax.format_ydata = perc

        self.set_legend(ax)
        self.format_ax(ax)

    def draw_custom_signals(self):
        """
        Draw custom signals on the chart.

        """
        ax = self.ax_custom_signals
        df = self.context.custom_signals_stats

        colors = ['blue', 'green', 'red', 'black', 'orange', 'yellow', 'pink']

        ax.clear()
        ax.set_title('Custom Signals')
        for index, column in enumerate(df.columns.values.tolist()):
            ax.plot(df.index, df[column], '-',
                    color=colors[index],
                    linewidth=1.0,
                    label=column
                    )

        self.set_legend(ax)
        self.format_ax(ax)

    def draw_exposure(self):
        """
        Draw exposure line on the chart.

        """
        ax = self.ax_exposure
        context = self.context
        df = context.exposure_stats

        # TODO: list exchanges in graph
        base_currency = None
        positions = []
        for exchange_name in context.exchanges:
            exchange = context.exchanges[exchange_name]

            if not base_currency:
                base_currency = exchange.base_currency
            elif base_currency != exchange.base_currency:
                raise MismatchingBaseCurrenciesExchanges(
                    base_currency=base_currency,
                    exchange_name=exchange.name,
                    exchange_currency=exchange.base_currency
                )

            positions += exchange.portfolio.positions

        ax.clear()
        ax.set_title('Exposure')
        ax.plot(df.index, df['base_currency'], '-',
                color='green',
                linewidth=1.0,
                label='Base Currency: {}'.format(base_currency.upper())
                )

        symbols = []
        for position in positions:
            symbols.append(position.symbol)

        ax.plot(df.index, df['long_exposure'], '-',
                color='blue',
                linewidth=1.0,
                label='Long Exposure: {}'.format(', '.join(symbols).upper()))

        self.set_legend(ax)
        self.format_ax(ax)

    def __iter__(self):
        yield pd.Timestamp.utcnow(), SESSION_START

        while True:
            current_time = pd.Timestamp.utcnow()
            current_minute = current_time.floor('1 min')

            if self._last_emit is None or current_minute > self._last_emit:
                log.debug('emitting minutely bar: {}'.format(current_minute))

                self._last_emit = current_minute
                yield current_minute, BAR

                try:
                    self.draw_pnl()
                    self.draw_custom_signals()
                    self.draw_exposure()

                    plt.draw()
                except Exception as e:
                    log.warn('Unable to update the graph: {}'.format(e))

            else:
                # I can't use the "animate" reactive approach here because
                # I need to yield from the main loop.

                # Workaround: https://stackoverflow.com/a/33050617/814633
                plt.pause(1)



from catalyst.exchange.bitfinex.bitfinex import Bitfinex
from catalyst.exchange.bittrex.bittrex import Bittrex
from catalyst.exchange.exchange_errors import ExchangeNotFoundError
from catalyst.exchange.exchange_utils import get_exchange_auth
from catalyst.exchange.poloniex.poloniex import Poloniex


def get_exchange(exchange_name, base_currency=None):
    exchange_auth = get_exchange_auth(exchange_name)
    if exchange_name == 'bitfinex':
        return Bitfinex(
            key=exchange_auth['key'],
            secret=exchange_auth['secret'],
            base_currency=base_currency,
            portfolio=None
        )

    elif exchange_name == 'bittrex':
        return Bittrex(
            key=exchange_auth['key'],
            secret=exchange_auth['secret'],
            base_currency=base_currency,
            portfolio=None
        )

    elif exchange_name == 'poloniex':
        return Poloniex(
            key=exchange_auth['key'],
            secret=exchange_auth['secret'],
            base_currency=base_currency,
            portfolio=None
        )

    else:
        raise ExchangeNotFoundError(exchange_name=exchange_name)


def get_exchanges(exchange_names):
    exchanges = dict()
    for exchange_name in exchange_names:
        exchanges[exchange_name] = get_exchange(exchange_name)

    return exchanges



#!/usr/bin/env python
import json
import time
import hmac
import hashlib
import ssl

# Workaround for backwards compatibility
# https://stackoverflow.com/questions/3745771/urllib-request-in-python-2-7
from six.moves import urllib

urlopen = urllib.request.urlopen


class Bittrex_api(object):
    def __init__(self, key, secret):
        self.key = key
        self.secret = secret
        self.public = ['getmarkets', 'getcurrencies', 'getticker',
                       'getmarketsummaries', 'getmarketsummary',
                       'getorderbook', 'getmarkethistory']
        self.market = ['buylimit', 'buymarket', 'selllimit', 'sellmarket',
                       'cancel', 'getopenorders']
        self.account = ['getbalances', 'getbalance', 'getdepositaddress',
                        'withdraw', 'getorder', 'getorderhistory',
                        'getwithdrawalhistory', 'getdeposithistory']

    def query(self, method, values={}):
        if method in self.public:
            url = 'https://bittrex.com/api/v1.1/public/'
        elif method in self.market:
            url = 'https://bittrex.com/api/v1.1/market/'
        elif method in self.account:
            url = 'https://bittrex.com/api/v1.1/account/'
        else:
            return 'Something went wrong, sorry.'

        url += method + '?' + urllib.parse.urlencode(values)

        if method not in self.public:
            url += '&apikey=' + self.key
            url += '&nonce=' + str(int(time.time()))

            signature = hmac.new(self.secret.encode('utf-8'),
                                 url.encode('utf-8'),
                                 hashlib.sha512).hexdigest()
            headers = {'apisign': signature}
        else:
            headers = {}

        req = urllib.request.Request(url, headers=headers)
        response = json.loads(urlopen(
            req, context=ssl._create_unverified_context()).read())

        if response["result"]:
            return response["result"]
        else:
            return response["message"]

    def getmarkets(self):
        return self.query('getmarkets')

    def getcurrencies(self):
        return self.query('getcurrencies')

    def getticker(self, market):
        return self.query('getticker', {'market': market})

    def getmarketsummaries(self):
        return self.query('getmarketsummaries')

    def getmarketsummary(self, market):
        return self.query('getmarketsummary', {'market': market})

    def getorderbook(self, market, type, depth=20):
        return self.query('getorderbook',
                          {'market': market, 'type': type, 'depth': depth})

    def getmarkethistory(self, market, count=20):
        return self.query('getmarkethistory',
                          {'market': market, 'count': count})

    def buylimit(self, market, quantity, rate):
        return self.query('buylimit', {'market': market, 'quantity': quantity,
                                       'rate': rate})

    def buymarket(self, market, quantity):
        return self.query('buymarket',
                          {'market': market, 'quantity': quantity})

    def selllimit(self, market, quantity, rate):
        return self.query('selllimit', {'market': market, 'quantity': quantity,
                                        'rate': rate})

    def sellmarket(self, market, quantity):
        return self.query('sellmarket',
                          {'market': market, 'quantity': quantity})

    def cancel(self, uuid):
        return self.query('cancel', {'uuid': uuid})

    def getopenorders(self, market):
        return self.query('getopenorders', {'market': market})

    def getbalances(self):
        return self.query('getbalances')

    def getbalance(self, currency):
        return self.query('getbalance', {'currency': currency})

    def getdepositaddress(self, currency):
        return self.query('getdepositaddress', {'currency': currency})

    def withdraw(self, currency, quantity, address):
        return self.query('withdraw',
                          {'currency': currency, 'quantity': quantity,
                           'address': address})

    def getorder(self, uuid):
        return self.query('getorder', {'uuid': uuid})

    def getorderhistory(self, market, count):
        return self.query('getorderhistory',
                          {'market': market, 'count': count})

    def getwithdrawalhistory(self, currency, count):
        return self.query('getwithdrawalhistory',
                          {'currency': currency, 'count': count})

    def getdeposithistory(self, currency, count):
        return self.query('getdeposithistory',
                          {'currency': currency, 'count': count})



"""
Requires Catalyst version 0.3.0 or above
Tested on Catalyst version 0.3.3

These example aims to provide and easy way for users to learn how to collect data from the different exchanges.
You simply need to specify the exchange and the market that you want to focus on.
You will all see how to create a universe and filter it base on the exchange and the market you desire.

The example prints out the closing price of all the pairs for a given market-exchange every 30 minutes.
The example also contains the ohlcv minute data for the past seven days which could be used to create indicators
Use this as the backbone to create your own trading strategies.

Variables lookback date and date are used to ensure data for a coin existed on the lookback period specified.
"""

import numpy as np
import pandas as pd
from datetime import timedelta
from catalyst import run_algorithm
from catalyst.exchange.exchange_utils import get_exchange_symbols

from catalyst.api import (
    symbols,
)


def initialize(context):
    context.i = -1  # counts the minutes
    context.exchange = 'poloniex'  # must match the exchange specified in run_algorithm
    context.base_currency = 'btc'  # must match the base currency specified in run_algorithm


def handle_data(context, data):
    lookback = 60 * 24 * 7  # (minutes, hours, days) of how far to lookback in the data history
    context.i += 1

    # current date formatted into a string
    today = context.blotter.current_dt
    date, time = today.strftime('%Y-%m-%d %H:%M:%S').split(' ')
    lookback_date = today - timedelta(days=(
    lookback / (60 * 24)))  # subtract the amount of days specified in lookback
    lookback_date = lookback_date.strftime('%Y-%m-%d %H:%M:%S').split(' ')[
        0]  # get only the date as a string

    # update universe everyday
    new_day = 60 * 24
    if not context.i % new_day:
        context.universe = universe(context, lookback_date, date)

    # get data every 30 minutes
    minutes = 30
    if not context.i % minutes and context.universe:
        # we iterate for every pair in the current universe
        for coin in context.coins:
            pair = str(coin.symbol)

            # 30 minute interval ohlcv data (the standard data required for candlestick or indicators/signals)
            # 30T means 30 minutes re-sampling of one minute data. change to your desire time interval.
            opened = fill(data.history(coin, 'open', bar_count=lookback,
                                       frequency='30T')).values
            high = fill(data.history(coin, 'high', bar_count=lookback,
                                     frequency='30T')).values
            low = fill(data.history(coin, 'low', bar_count=lookback,
                                    frequency='30T')).values
            close = fill(data.history(coin, 'price', bar_count=lookback,
                                      frequency='30T')).values
            volume = fill(data.history(coin, 'volume', bar_count=lookback,
                                       frequency='30T')).values

            # close[-1] is the equivalent to current price
            # displays the minute price for each pair every 30 minutes
            print(
            today, pair, opened[-1], high[-1], low[-1], close[-1], volume[-1])

            # ----------------------------------------------------------------------------------------------------------
            # -------------------------------------- Insert Your Strategy Here -----------------------------------------
            # ----------------------------------------------------------------------------------------------------------


def analyze(context=None, results=None):
    pass


# Get the universe for a given exchange and a given base_currency market
# Example: Poloniex btc Market
def universe(context, lookback_date, current_date):
    json_symbols = get_exchange_symbols(
        context.exchange)  # get all the pairs for the exchange
    universe_df = pd.DataFrame.from_dict(json_symbols).transpose().astype(
        str)  # convert into a dataframe
    universe_df['base_currency'] = universe_df.apply(
        lambda row: row.symbol.split('_')[1],
        axis=1)
    universe_df['market_currency'] = universe_df.apply(
        lambda row: row.symbol.split('_')[0],
        axis=1)
    # Filter all the exchange pairs to only the ones for a give base currency
    universe_df = universe_df[
        universe_df['base_currency'] == context.base_currency]

    # Filter all the pairs to ensure that pair existed in the current date range
    universe_df = universe_df[universe_df.start_date < lookback_date]
    universe_df = universe_df[universe_df.end_daily >= current_date]
    context.coins = symbols(
        *universe_df.symbol)  # convert all the pairs to symbols
    return universe_df.symbol.tolist()


# Replace all NA, NAN or infinite values with its nearest value
def fill(series):
    if isinstance(series, pd.Series):
        return series.replace([np.inf, -np.inf], np.nan).ffill().bfill()
    elif isinstance(series, np.ndarray):
        return pd.Series(series).replace([np.inf, -np.inf],
                                         np.nan).ffill().bfill().values
    else:
        return series


if __name__ == '__main__':
    start_date = pd.to_datetime('2017-01-08', utc=True)
    end_date = pd.to_datetime('2017-11-13', utc=True)

    performance = run_algorithm(start=start_date, end=end_date,
                                capital_base=10000.0,
                                initialize=initialize,
                                handle_data=handle_data,
                                analyze=analyze,
                                exchange_name='poloniex',
                                data_frequency='minute',
                                base_currency='btc',
                                live=False,
                                live_graph=False,
                                algo_namespace='simple_universe')

"""
Run in Terminal (inside catalyst environment):
python simple_universe.py
"""
